//
//  AviConverterWindowController.java
//  CocoDonkey
//  $Id: AviConverterWindowController.java,v 1.3 2002/06/09 21:02:31 fortun Exp $
//
//  Created by Fred Bonzoun on Mon May 20 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.io.*;

/***
 * Controls a window that converts all the AVI files in the incoming directory to Quicktime
 * movies with the help of the Avi2Mov application. An AVI file is a file with an extension of
 * "avi" or "divx"
 * The window displays a progress bar controlled by a separate thread.
 * <P>It opens a window only if there is some AVI files to convert. Otherwise, it just dies silently.
 ***/
public class AviConverterWindowController extends WindowController {

    NSTextField nameField; /* IBOutlet */
    NSProgressIndicator progressBar; /* IBOutlet */
    NSTextField statusField; /* IBOutlet */
    /** Button linked to the closeWindow action **/
    NSButton previewButton; /* IBOutlet */

    /** File being converted **/
    private File source;
    /** Current file converted **/
    private File dest;
    /** Avi2Mov process doing the conversion **/
    private Process transfert;
    /** Thread listening to the convertion process and updating the progress bar **/
    private Worker worker;
    /** The path to the Avi2Mov application **/
    private String avi2mov;

    /***
     * Load the nib and shows the window if there is something to do
     ***/
    public AviConverterWindowController() {
        super("AviConverterWindow");
        avi2mov = NSBundle.mainBundle().pathForResource("Avi2Mov", null);
    }

    /***
     * Shows the window if there is something to be converted
     ***/
    public void windowDidLoad() {
        super.windowDidLoad();
        if (convertOneFile())
            showWindow(window());
        else
            window().close();
    }

    /***
     * Launches the process to convert an AVI and the thread that listens to it.
     * @return True if there is something to be converted, false otherwise
     ***/
    private synchronized boolean convertOneFile() {
        // We clean some instance variables
        transfert = null;
        worker = null;

        // We get the location of the file
        source = nextFileToConvert();
        if (source==null)
            return false;
        // We convert it
        dest = new File(source.getParent(), movFileName(source.getName()));
        try {
            transfert = Runtime.getRuntime().exec(new String[]{avi2mov, source.getAbsolutePath(), dest.getAbsolutePath()});
        }
        catch(IOException e) {
            cleanAndDestroy();
            return false;
        }

        // We initialize the GUI
        nameField.setStringValue(source.getName());
        statusField.setStringValue(Messages.string("Converting..."));
        progressBar.setIndeterminate(false);
        progressBar.setDoubleValue(0);
        progressBar.setMaxValue(source.length());

        // We launch the listener
        worker = new Worker();
        worker.start();

        return true;
    }

    /***
     * Returns the first AVI file found in the "incoming" directory.
     * <P>NB: the "incoming" name should be taken from the core and not hardcoded
     ***/
    public static File nextFileToConvert() {
        // We get the location of the file
        File dir = new File(Preferences.daemonLocation(), "incoming");
        if (!dir.exists())
            return null;
        String[] files = dir.list();
        for(int i=0; i<files.length; i++) {
            String name = files[i];
            String ext = extension(name);
            // We only translate avi files
            if (!ext.equalsIgnoreCase("avi") && !ext.equalsIgnoreCase("divx"))
                continue;
            File dest = new File(dir, movFileName(name));
            if (dest.exists())
                continue;
            // We convert it
            return new File(dir, name);
        }
        return null;
    }

    /***
     * @return The extension of the parameter, "" if none is found
     ***/
    private static String extension(String name) {
        int p = name.lastIndexOf('.');
        if (p<0)
            return "";
        return name.substring(p+1);
    }

    /***
     * Converts the given filename to a valid Quicktime 31 chars name.
     * It changes the extension to ".mov" and reduces the file length to 31 maximum.
     * The length is reduced because Avi2Mov does not accept names greater than 31 (old HFS)
     ***/
    private static String movFileName(String name) {
        int p = name.lastIndexOf('.');
        if (p<0)
            return "";
        String dname = name.substring(0, p);
        // Avi2Mov only accepts 31 chars for the file names
        if (dname.length()>23)
            dname = dname.substring(0, 23) + "... ";
        return dname + ".mov";
    }

    /***
     * Closes the window. If a conversion was running, it will be cleared by the windowWillClose event
     ***/
    public void closeWindow(Object sender) { // IBAction
        window().close();
    }

    /***
     * Stops the thread and the conversion process, deletes the currently being converted file if any.
     ***/
    private synchronized void clean() {
        if (worker!=null)
            worker.stopRunning();
        if (transfert!=null) {
            transfert.destroy();
            transfert = null;
            if (dest!=null)
                dest.delete();
        }
    }

    /***
     * Cleans and then ensures that the the currently being converted file is deleted.
     ***/
    private void cleanAndDestroy() {
        clean();
        if (dest!=null)
            dest.delete();
    }

    /***
     * Cleans, ie calls the clean() method
     ***/
    public void windowWillClose(NSNotification aNotification) {
        super.windowWillClose(aNotification);
        clean();
    }

    /***
     * Listens to the conversion process to update the progress bar
     ***/
    private class Worker extends WorkerThread {
        protected boolean work() throws Exception {
            progressBar.setDoubleValue(dest.length());
            try {
                int cr = 1;
                // We synchronize to avoide conflict with the close of the window
                synchronized(AviConverterWindowController.this) {
                    if (transfert!=null) {
                        // This launches an IllegalThreadStateException if the process is still running
                        cr = transfert.exitValue();
                        transfert = null;
                    }
                    // We check the exit status, 0 means OK
                    if (cr==0) {
                        // mov is smaller than avi, so ...
                        progressBar.setDoubleValue(source.length());
                        statusField.setStringValue(Messages.string("Converted"));
                        // Transfert Finished
                        Runtime.getRuntime().exec(new String[]{"open", dest.getAbsolutePath()});
                        convertOneFile();
                    } else {
                        statusField.setStringValue(Messages.string("Error: %@", cr));
                        worker = null;
                    }
                }
                // Process not running, so must the thread
                return false;
            }
            catch(IllegalThreadStateException e) {
                // Process still running, so must the trhrad
                return true;
            }
        }
    }

}

// $Log: AviConverterWindowController.java,v $
// Revision 1.3  2002/06/09 21:02:31  fortun
// Comments added
// Bug is several avi files need to be translated
//
// Revision 1.2  2002/06/01 15:15:02  fortun
// Better stop of the WorkerThread
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
