//
//  DaemonOptionsPanelController.java
//  CocoDonkey
//  $Id: DaemonOptionsPanelController.java,v 1.25 2002/08/21 21:40:40 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import net.bonzoun.cocodonkey.*;
import java.util.*;
import java.io.*;
import gnu.regexp.*;

/***
 * Controls the DaemonOptionsPanel panel.
 ***/
public class DaemonOptionsPanelController extends WindowController {

    /** regular expression that can recognize a hash from edonkey describing a file **/
    private static RE reHash;

    static {
        try {
            reHash = new RE("^[0-9,A-F]{32}$");
        }
        catch(REException e) {
            e.printStackTrace();
        }
    }
    
    NSTextField downloadRate; /* IBOutlet */
    NSTextField uploadRate; /* IBOutlet */
    NSTextField uploadCommentField; /* IBOutlet */
    NSTextField uploadCreditField; /* IBOutlet */
    NSTableView serversList; /* IBOutlet */
    NSTextField serversComment; /* IBOutlet */
    NSTextField maxServers; /* IBOutlet */
    NSMatrix autoUpdateServers; /* IBOutlet */
    NSTextView consoleView; /* IBOutlet */
    NSTableView allServersList; /* IBOutlet */
    NSTextField allServersComment; /* IBOutlet */
    NSButton connectButton; /* IBOutlet */
    NSButton restartButton; /* IBOutlet */
    NSButton serverMetButton; /* IBOutlet */

    private DonkeyTalker talker;
    private DownloadItemSource dataSource;
    /** Object that handle the sorting of the servers list **/
    private TableViewSorter serversListSorter;
    private TableViewSorter allServersListSorter;
    /** Last offset in the console data **/
    private int lastConsoleIndex;
    
    private Worker thread;
    /** List of the servers the core is connected to **/
    private NSMutableDictionary servers = new NSMutableDictionary();
    private List allServers = new ArrayList();

    private boolean calcConnectedServers;
    private boolean calcKnownServers;

    /***
     * Loads the nib, but does not show the window
     ***/
    public DaemonOptionsPanelController(DonkeyTalker talker, DownloadItemSource source) {
        super("DaemonOptionsPanel");
        this.talker = talker;
        dataSource = source;
    }

    private void setControlValueIfNotNull(NSControl ctrl, String value) {
        if (value!=null)
            ctrl.setStringValue(value);
        else
            ctrl.setStringValue("?");
    }
    
    /***
     * Updates the display and launches the thread that will update it
     ***/
    public void windowDidLoad() {
        super.windowDidLoad();
        lastConsoleIndex = talker.currentDialogOffset();

        calcConnectedServers = false;
        calcKnownServers = false;

        if (!talker.processIsLocal()) {
            restartButton.setEnabled(false);
            serverMetButton.setEnabled(false);
        }
        
        NSDictionary options = talker.options();
        setControlValueIfNotNull(uploadRate, (String)options.objectForKey("max_hard_upload_rate"));
        setControlValueIfNotNull(downloadRate, (String)options.objectForKey("max_hard_download_rate"));
        // This option is sometimes called "update_server_list" (core 1.15) or "dont_update_server_list" (core  1.16)
        String opt = (String)options.objectForKey("update_server_list");
        if (opt==null) {
            opt = (String)options.objectForKey("dont_update_server_list");
            if (opt!=null && opt.equalsIgnoreCase("false"))
                autoUpdateServers.selectCellWithTag(1);
            else
                autoUpdateServers.selectCellWithTag(0);
        } else {
            if (opt!=null && opt.equalsIgnoreCase("true"))
                autoUpdateServers.selectCellWithTag(1);
            else
                autoUpdateServers.selectCellWithTag(0);
        }
        setControlValueIfNotNull(maxServers, (String)options.objectForKey("max_connected_servers"));
        // updateCredits();

        // We initialize the sorting object
        serversListSorter.addSortedColumn("Name", new ServerItem.CompareName());
        serversListSorter.addSortedColumn("IP", new ServerItem.CompareIP());
        serversListSorter.addSortedColumn("N1", new ServerItem.CompareN1());
        serversListSorter.addSortedColumn("N2", new ServerItem.CompareN2());
        allServersListSorter.addSortedColumn("Name", new ServerItem.CompareName());
        allServersListSorter.addSortedColumn("IP", new ServerItem.CompareIP());

        // We launch the GUI update thread
        thread = new Worker();
        thread.start();
    }

    /** Updates the UpdateCredit and UploadCreditComment fields **/
    private void updateCredits() {
        uploadCreditField.setStringValue("" + talker.uploadCredit() + "m");
        int n = talker.uploadDisabledTime();
        if (n>0)
            uploadCommentField.setStringValue(Messages.string("Upload disabled for %@ minutes", n));
        else
            uploadCommentField.setStringValue(Messages.string("<No upload limit>"));
    }

    public void useAllUploadCredit(Object sender) { // IBAction
        int n = uploadCreditField.intValue();
        if (n>=5) {
            talker.disableUploadFor(n - (n%5));
            updateCredits();
        }
    }

    public void useHalfUploadCredit(Object sender) { // IBAction
        int n = uploadCreditField.intValue() / 2;
        if (n>=5) {
            talker.disableUploadFor(n - (n%5));
            updateCredits();
        }
    }

    public void use15UploadCredit(Object sender) { // IBAction
        talker.disableUploadFor(15);
        updateCredits();
    }

    /** Unused **/
    public void cancel(Object sender) { // IBAction
        // NSApplication.sharedApplication().stopModalWithCode(0);
        // window().close();
    }

    /** Saves the options to the core **/
    public void save(Object sender) { // IBAction
        talker.setOption("max_hard_upload_rate", uploadRate.stringValue());
        talker.setOption("max_hard_download_rate", downloadRate.stringValue());
        talker.setOption("max_connected_servers", maxServers.stringValue());
        if (autoUpdateServers.selectedCell()!=null && autoUpdateServers.selectedCell().tag()==1) {
            talker.setOption("update_server_list", "true");
            talker.setOption("dont_update_server_list", "false");
        } else {
            talker.setOption("update_server_list", "false");
            talker.setOption("dont_update_server_list", "true");
        }
        
        // NSApplication.sharedApplication().stopModalWithCode(1);
        // window().close();
    }

    /** Ask the core to add servers **/
    public void addServers(Object sender) { // IBAction
        talker.addServers();
    }

    /** Opens the window that allows to add a given server to the core **/
    public void addAServer(Object sender) { // IBAction
        AddServerPanelController controller = new AddServerPanelController(talker);
        // controller.window().setTitle(controller.window().title() + " - " + window().title());
        controller.showWindow(controller.window());
    }

    /** Adds a list of servers to the daemon (a server.met file) **/
    public void addServerMet(Object sender) { // IBAction
        NSOpenPanel panel = NSOpenPanel.openPanel();
        int cr = panel.runModalInDirectory(null, "server.met", new NSArray(new String[]{"met"}));
        if (cr == NSOpenPanel.OKButton) {
            String file = (String)panel.filenames().objectAtIndex(0);
            talker.addServerMet(file);
        }
    }
    
	/** Opens the window that allows to add a given server to the core **/
	public void connectServer(Object sender) { // IBAction
        java.util.List list = allServersListSorter.list();
        NSEnumerator enum = allServersList.selectedRowEnumerator();
        while (enum.hasMoreElements()) {
            Number row = (Number)enum.nextElement();
            ServerItem item = (ServerItem)list.get(row.intValue());
            talker.connectToServer(String.valueOf(item.id()));
        }
    }

    /***
     * Deletes "by hand" the files that have been canceled.
     * Often, the core removes the file but does not really delete it
     ***/
    public void emptyTrash(Object sender) { // IBAction
        // We close the files
        talker.reshare();
        talker.closeFds();
        // We get the files from the temp directory
        /*File dir = new File(Preferences.daemonLocation() + "/temp");
        NSMutableArray files = new NSMutableArray(dir.list());
        for(int i=files.count()-1; i>=0; i--) {
            String filename = (String)files.objectAtIndex(i);
            if (reHash.getMatch(filename)==null)
                files.removeObjectAtIndex(i);
        }
        // We subtract from this list the files currently downloading
        for(int i=0; i<dataSource.itemsCount(); i++) {
            DownloadItem item = dataSource.itemAtRow(i);
            if (item==null)
                continue;
            if (item.hash()==null)
                talker.fillItemDetails(item);
            if (item.hash()==null)
                return;
            int n = files.indexOfObject(item.hash());
            if (n!=NSArray.NotFound)
                files.removeObjectAtIndex(n);
        }
        // We finally delete the files
        for(int i=0; i<files.count(); i++) {
            if (!new File(dir, (String)files.objectAtIndex(i)).delete())
                System.out.println("## Cannot delete file " + files.objectAtIndex(i));
        }*/
    }

    /** Recovers from the temp directory the files that are there but are not in the list **/
    public void recoverTrash(Object sender) { // IBAction
        talker.recoverTrash();
    }

    /** Asks the server to remove the unused servers **/
    public void removeOldServers(Object sender) { // IBAction
        talker.removeOldServers();
    }

    /** Tries to kill and then to restart the core **/
    public void restartDaemon(Object sender) { // IBAction
        String hostname = talker.hostname();
        if (!killCore(talker))
            return;
        // We wait a little, to give some time to the system to rest
        try {
            Thread.sleep(1000);
        }
        catch(InterruptedException e) {
        }
        // Now we start the core
        try {
            if (hostname==null) {
                DonkeyLauncher.launchLocalDonkey(Preferences.daemonLocation(), talker);
            } else {
                // To do
            }
	        // talker.reconnect();
        }
        catch(IOException e) {
            NSAlertPanel.runCriticalAlert(Messages.string("Unable to launch local Donkey"),
                                          e.getMessage(),
                                          Messages.string("OK"), null, null);
        }
    }

    public static boolean killCore(DonkeyTalker talker) {
        // We synchronize to be sure that nobody will try to relaunch the daemon
        synchronized (talker) {
            // We soft-kill the core
            if (talker.kill()) {
                return true;
            } else {
                int cr = NSAlertPanel.runInformationalAlert(Messages.string("Local Donkey still running"),
                                    Messages.string("Do you want to kill the local daemon ?"),
                                    Messages.string("No"), Messages.string("Yes"), null);
                if (cr == NSAlertPanel.AlternateReturn) {
                    if (DonkeyLauncher.hardKillCore()) {
                        return true;
                    } else {
                        NSAlertPanel.runInformationalAlert(Messages.string("Local Donkey still running"),
                                    Messages.string("Could not stop the local donkey"),
                                    "OK", null, null);
                        return false;
                    }
                } else {
                    return false;
                }
            }
        }
    }
    
    /** Closes the window and stops the update thread **/
    public void windowWillClose(NSNotification aNotification) {
        super.windowWillClose(aNotification);
        if (calcConnectedServers)
            Preferences.rememberColumnSizes(this, serversList);
        if (calcKnownServers)
            Preferences.rememberColumnSizes(this, allServersList);
        talker.stopCommandIfCurrent("vma");
        if (thread!=null)
            thread.stopRunning();
    }

    /** For the servers list. **/
    public int numberOfRowsInTableView(NSTableView aTableView) {
        if (aTableView==serversList)
            return servers.count();
        else
            return allServers.size();
    }

    /** For the servers list. **/
    public final Object tableViewObjectValueForLocation(NSTableView aTableView, NSTableColumn aTableColumn, int rowIndex) {
        ServerItem item;
        if (aTableView==serversList)
            item = (ServerItem)serversListSorter.list().get(rowIndex);
        else
            item = (ServerItem)allServersListSorter.list().get(rowIndex);
        String name = (String)aTableColumn.identifier();
        if (name.equals("Name"))
            return item.name();
        if (name.equals("IP"))
            return item.ip();
        if (name.equals("N1"))
            return new Integer(item.n1());
        if (name.equals("N2"))
            return new Integer(item.n2());
        return "?";
    }

    /** For the servers list. Sorts the list **/
    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        if (tableView==serversList)
            serversListSorter.tableViewDidClickTableColumn(tableView, tableColumn);
        else
            allServersListSorter.tableViewDidClickTableColumn(tableView, tableColumn);
    }

	public void tableViewSelectionDidChange(NSNotification aNotification) {
        NSTableView table = (NSTableView)aNotification.object();
        if (table==allServersList) {
            int row = allServersList.selectedRow();
            connectButton.setEnabled(row>=0);
        }
    }

	public void tabViewDidSelectTabViewItem( NSTabView tabView, NSTabViewItem tabViewItem) {
        calcConnectedServers = false;
        if (!calcConnectedServers && tabViewItem.identifier().equals("ConnectedServers")) {
            if (Preferences.rememberWindowPos())
                Preferences.restoreColumnSizes(this, serversList);
            calcConnectedServers = true;
            thread.wakeUp();
        } else if (!calcKnownServers && tabViewItem.identifier().equals("KnownServers")) {
            if (Preferences.rememberWindowPos())
                Preferences.restoreColumnSizes(this, allServersList);
            calcKnownServers = true;
            thread.wakeUp();
        }
    }

    /** This thread updates the values in the window **/
    private class Worker extends WorkerThread {

        private void updateList(NSArray liste, NSMutableDictionary servers, TableViewSorter sorter, NSTableView table) {
            //boolean reload = false;
            for(int i=0; i<liste.count(); i++) {
                ServerItem server = (ServerItem)liste.objectAtIndex(i);
                Number id = new Integer(server.id());
                ServerItem orig = (ServerItem)servers.objectForKey(id);
                if (orig==null) {
                    // We add a new server
                    servers.setObjectForKey(server, id);
                    sorter.addObject(server);
                    //reload = true;
                } else {
                    // We update the values of a known server
                    orig.updateWith(server);
                    sorter.updateObject(orig);
                }
            }
            //if (reload)
            //    table.reloadData();
            //table.setNeedsDisplay(true);
        }
        
        public void update(NSTimer timer) {
            updateList((NSArray)timer.userInfo(), servers, serversListSorter, serversList);
        }

        public void update2(NSTimer timer) {
            //updateList((NSArray)timer.userInfo(), allServers, allServersListSorter, allServersList);
            allServers = (List)timer.userInfo();
            allServersListSorter.setList(allServers);
        }

        private int loopConnectedServers;
        private int loopKnownServers;
        
        public boolean work() throws Exception {
            updateCredits();
            if (calcConnectedServers) {
                if ( (loopConnectedServers++%2) == 0 ) {
                    NSArray liste = talker.serversList();
                    serversComment.setStringValue(Messages.string("%@ servers connected", liste.count()));
                    doTask("update", liste);
                }
            }
            String s = talker.dialogSince(lastConsoleIndex);
            consoleView.textStorage().appendAttributedString(new NSAttributedString(s));
            lastConsoleIndex += s.length();
            if (calcKnownServers) {
                if ( (loopKnownServers++%10) == 0 ) {
                    List liste2 = talker.allServersList();
                    allServersComment.setStringValue(Messages.string("%@ known servers", liste2.size()));
                    doTask("update2", liste2);
                }
            }
            return true;
        }
    }
}

// $Log: DaemonOptionsPanelController.java,v $
// Revision 1.25  2002/08/21 21:40:40  fortun
// Bug on console offset initialization
//
// Revision 1.24  2002/07/28 19:47:15  fortun
// Some buttons are grayed if the daemon is not local
// New addServerMet() functionality
// The "all servers list" should display faster
//
// Revision 1.23  2002/07/28 18:48:13  fortun
// Bug on column sizes remember
//
// Revision 1.22  2002/07/25 23:35:27  fortun
// Now the display updates are done in the TableSorter
//
// Revision 1.21  2002/07/22 00:30:23  fortun
// NSArrays are now Lists
//
// Revision 1.20  2002/07/14 23:27:23  fortun
// Remembers the columns position and size
//
// Revision 1.19  2002/07/14 22:34:02  fortun
// emptyTrash() reshares before closing the files (should remove unused files)
// allServers() serach stopped when closing
// wakeUp() called when changing tab only the first time
// Lists updated not so often
//
// Revision 1.18  2002/07/09 22:07:08  fortun
// Handles the new list of all known servers
// Optimizes the CPU by not calcultating the not opened tabs
// killCore() is back here
// reloadData() call on the lists is optimized
//
// Revision 1.17  2002/07/08 22:14:54  fortun
// Message if not able to kill core is now in the killCore() method
//
// Revision 1.16  2002/07/08 21:54:03  fortun
// Solved bug if an option is unavailable in the daemon
//
// Revision 1.15  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.14  2002/07/07 19:38:44  fortun
// Now handles the selection of the header
//
// Revision 1.13  2002/07/07 12:39:46  fortun
// API syntax change
//
// Revision 1.12  2002/07/04 23:53:09  fortun
// New Logger parameter to launch the core
//
// Revision 1.11  2002/07/04 23:13:13  fortun
// Now uses DonkeyLauncher to kill the core, which uses a soft kill and then a hard kill
//
// Revision 1.10  2002/06/30 21:00:12  fortun
// List update is now done in the main thread
//
// Revision 1.9  2002/06/29 17:52:59  fortun
// Added some locks to prevent problems with the GUI
//
// Revision 1.8  2002/06/14 00:35:00  fortun
// Gestion de la console
//
// Revision 1.7  2002/06/09 21:03:34  fortun
// Comments added
//
// Revision 1.6  2002/06/08 12:15:50  fortun
// Error when terminating if no thread is running
//
// Revision 1.5  2002/06/02 22:17:55  fortun
// Bug with update_server_list option that changed from 1.15 to 1.16 of the core
//
// Revision 1.4  2002/06/01 15:15:02  fortun
// Better stop of the WorkerThread
//
// Revision 1.3  2002/05/27 22:00:14  fortun
// New 'maxServers', 'autoUpdate' and 'addAServer' options
// Uses the new TableViewSorter to sort the list
//
// Revision 1.2  2002/05/25 23:09:02  fortun
// on restart, only relaunch if local donkey
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
