//
//  MainWindowToolbar.java
//  CocoDonkey
//  $Id: MainWindowToolbar.java,v 1.5 2002/07/28 19:34:38 fortun Exp $
//
//  Created by Fred Bonzoun on Thu May 16 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;


public class MainWindowToolbar extends NSToolbar {

    private MainWindowController windowController;
    private NSMutableArray items = new NSMutableArray();
    
    public MainWindowToolbar(MainWindowController controller) {
        super("main");
        windowController = controller;
        setDelegate(this);
        setAutosavesConfiguration(true);
        setAllowsUserCustomization(true);
        setDisplayMode(NSToolbar.NSToolbarDisplayModeIconOnly);
    }

    public void willClose() {
        windowController = null;
    }
    
    public NSToolbarItem toolbarItemForItemIdentifier(NSToolbar toolbar, String itemIdentifier, boolean flag) {
        NSToolbarItem item;
        if (itemIdentifier.equals("start")) {
            item = new OnSelectionItem(itemIdentifier);
            item.setAction(new NSSelector("resumeDownload", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Resume"));
            item.setToolTip(Messages.string("Resumes the download of the selected line"));
            item.setImage(makeImageFor("StartOn"));
        }
        else if (itemIdentifier.equals("pause")) {
            item = new OnSelectionItem(itemIdentifier);
            item.setAction(new NSSelector("pauseDownload", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Pause"));
            item.setToolTip(Messages.string("Pauses the download of the selected line"));
            item.setImage(makeImageFor("PauseOn"));
        }
        else if (itemIdentifier.equals("cancel")) {
            item = new OnSelectionItem(itemIdentifier);
            item.setAction(new NSSelector("cancelDownload", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Cancel"));
            item.setToolTip(Messages.string("Cancels the download of the selected line"));
            item.setImage(makeImageFor("DeleteOn"));
        }
        else if (itemIdentifier.equals("commit")) {
            item = new OnHasFinished(itemIdentifier);
            item.setAction(new NSSelector("commit", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Commit Downloads"));
            item.setToolTip(Messages.string("Commits all the finished downloads in the list"));
            item.setImage(makeImageFor("CommitOn"));
        }
        else if (itemIdentifier.equals("info")) {
            item = new OnDrawerOpened(itemIdentifier);
            item.setAction(new NSSelector("toggleInfoPanel", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Show Infos"));
            item.setToolTip(Messages.string("Shows/hides the infos for the current line"));
            item.setImage(makeImageFor("InformationOn"));
        }
        else if (itemIdentifier.equals("search")) {
            item = new NSToolbarItem(itemIdentifier);
            item.setAction(new NSSelector("startSearch", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Search"));
            item.setToolTip(Messages.string("Opens the search window"));
            item.setImage(makeImageFor("SearchOn"));
        }
        else if (itemIdentifier.equals("options")) {
            item = new NSToolbarItem(itemIdentifier);
            item.setAction(new NSSelector("openOptions", new Class[]{Object.class}));
            item.setPaletteLabel(Messages.string("Daemon Options"));
            item.setToolTip(Messages.string("Opens the window for the daemon options"));
            item.setImage(makeImageFor("OptionsOn"));
        }
        else if (itemIdentifier.equals("ed2k")) {
            item = new NSToolbarItem(itemIdentifier);
            item.setAction(new NSSelector("ed2kLink", new Class[]{Object.class}));
            item.setPaletteLabel("ed2k");
            item.setToolTip(Messages.string("Downloads based on an ed2k link"));
            item.setImage(makeImageFor("Ed2kLinkOn"));
        }
        else if (itemIdentifier.equals("incoming")) {
            if (windowController.processIsLocal())
                item = new NSToolbarItem(itemIdentifier);
            else
                item = new DisabledItem(itemIdentifier);
            item.setAction(new NSSelector("openIncoming", new Class[]{Object.class}));
            item.setPaletteLabel("Show Incoming");
            item.setToolTip(Messages.string("Opens in the finder the folder containing the downloaded files"));
            item.setImage(makeImageFor("IncomingOn"));
        }
        else
            return null;
        item.setLabel(item.paletteLabel());
        item.setTarget(windowController);
        items.addObject(item);
        return item;
    }

    private NSImage makeImageFor(String name) {
        name = NSBundle.mainBundle().pathForResource(name, "tif");
        return new NSImage(name, false);
    }
    
    public NSArray toolbarAllowedItemIdentifiers(NSToolbar toolbar) {
        return new NSArray(new String[]{
            "start", "pause", "cancel", "commit", "info", "search", "options", "ed2k", "incoming", NSToolbarItem.SeparatorItemIdentifier, NSToolbarItem.SpaceItemIdentifier, NSToolbarItem.FlexibleSpaceItemIdentifier
        });
    }

    public NSArray toolbarDefaultItemIdentifiers(NSToolbar toolbar) {
        return new NSArray(new String[]{
            "options", "search", "ed2k", NSToolbarItem.FlexibleSpaceItemIdentifier, "info", "start", "pause", "cancel", NSToolbarItem.FlexibleSpaceItemIdentifier, "commit", "incoming"
        });
    }

    public class DisabledItem extends NSToolbarItem {
        public DisabledItem(String itemName) {
            super(itemName);
        }

        public void validate() {
            setEnabled(false);
        }
    }
    
    public class OnSelectionItem extends NSToolbarItem {
        public OnSelectionItem(String itemName) {
            super(itemName);
        }
        
        public void validate() {
            setEnabled(windowController.hasSelectedItem());
        }
    }

    public class OnHasFinished extends NSToolbarItem {
        public OnHasFinished(String itemName) {
            super(itemName);
        }

        public void validate() {
            setEnabled(windowController.hasFinishedItems());
        }
    }

    public class OnDrawerOpened extends NSToolbarItem {
        public OnDrawerOpened(String itemName) {
            super(itemName);
        }

        public void validate() {
            setEnabled(windowController.isDrawerOpened() || windowController.hasSelectedItem());
        }
    }
}

// $Log: MainWindowToolbar.java,v $
// Revision 1.5  2002/07/28 19:34:38  fortun
// Incoming button is now disabled if not local
//
// Revision 1.4  2002/07/14 22:37:13  fortun
// willClose() method added to ease the task of the gc
//
// Revision 1.3  2002/06/09 15:40:12  fortun
// Added text under the icons
//
// Revision 1.2  2002/05/25 23:16:13  fortun
// Handles the enable/disable state of the items
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
