//
//  SearchWindowController.java
//  CocoDonkey
//  $Id: SearchWindowController.java,v 1.13 2002/07/30 00:16:37 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import net.bonzoun.cocodonkey.*;
import java.util.*;

public class SearchWindowController extends WindowController {

    NSTextField aliasesField; /* IBOutlet */
    NSTextField codecField; /* IBOutlet */
    SearchedItemSource dataSource; /* IBOutlet */
    NSTextField identifierField; /* IBOutlet */
    NSDrawer infoDrawer; /* IBOutlet */
    NSProgressIndicator progressBar; /* IBOutlet */
    NSTableView resultList; /* IBOutlet */
    NSComboBox searchField; /* IBOutlet */
    NSButton searchButton; /* IBOutlet */
    NSButton downloadButton; /* IBOutlet */
    NSButton extendedButton; /* IBOutlet */

    NSDrawer extendedSearchDrawer; /* IBOutlet */
    NSTextField asAlbumField; /* IBOutlet */
    NSTextField asArtistField; /* IBOutlet */
    NSTextField asFormatField; /* IBOutlet */
    NSTextField asMaxSizeField; /* IBOutlet */
    NSPopUpButton asMaxSizeTypeButton; /* IBOutlet */
    NSTextField asMediaField; /* IBOutlet */
    NSPopUpButton asMinBitrateButton; /* IBOutlet */
    NSTextField asMinSizeField; /* IBOutlet */
    NSPopUpButton asMinSizeTypeButton; /* IBOutlet */
    NSTextField asTitleField; /* IBOutlet */
    
    private DonkeyTalker talker;
    private DownloadItemSource downloadItemSource;
    private int currentSearch;
    private boolean mustSearch = true;
    private NSArray oldResults = new NSArray();
    private ItemSearch thread = null;
    
    public SearchWindowController(DonkeyTalker talker, DownloadItemSource source) {
        super("SearchWindow");
        this.talker = talker;
        downloadItemSource = source;
    }

    public void windowDidLoad() {
        super.windowDidLoad();
        if (Preferences.rememberWindowPos())
            Preferences.restoreColumnSizes(this, resultList);
        //infoDrawer.setContentSize(new NSSize(200, 170));
        searchField.setHasVerticalScroller(true);
        dataSource.setDownloadItemSource(downloadItemSource);
        dataSource.windowDidLoad();
    }

    public void extendedClicked(Object sender) {
        if (extendedButton.state() == NSCell.OnState) {
            extendedSearchDrawer.open();
        } else {
            extendedSearchDrawer.close();
        }
    }

    public synchronized void extendSearch(Object sender) {
        talker.extendSearch();
        if (currentSearch>0 && (thread==null || !thread.isRunning()))
            startSearch();
    }
    
    public synchronized void search(Object sender) {
        if (mustSearch) {
            String pattern = searchField.stringValue().trim();
            if (extendedSearchDrawer.state()==NSDrawer.ClosedState || extendedSearchDrawer.state()==NSDrawer.ClosingState) {
                // Simple search
                if (pattern.length()>0) {
                    int[] res = talker.startQuery(pattern);
                    currentSearch = res[0];
                    dataSource.setElements(null);
                    startSearch();
                }
            } else {
                // Extended search
                int minSize = asMinSizeField.intValue() * asMinSizeTypeButton.selectedItem().tag();
                int maxSize = asMaxSizeField.intValue() * asMaxSizeTypeButton.selectedItem().tag();
                String media = asMediaField.stringValue().trim();
                String format = asFormatField.stringValue().trim();
                String artist = asArtistField.stringValue().trim();
                String album = asAlbumField.stringValue().trim();
                String title = asTitleField.stringValue().trim();
                int minBitrate = asMinBitrateButton.selectedItem().tag();
                int[] res = talker.startQuery(pattern, minSize, maxSize, media, format, artist, album, title, minBitrate);
                currentSearch = res[0];
                dataSource.setElements(null);
                startSearch();                
            }
        } else {
            int oldSearch = currentSearch;
            // We stop the running thread, if any
            currentSearch = -1;
            if (thread!=null) {
                thread.stopRunningImmediately();
                thread = null;
            }
            // THEN, we forget the search (because it can be blocked by the thead we stop)
            talker.forget(oldSearch);
            // We are ready for a new search
            mustSearch = true;
            searchButton.setTitle(Messages.string("Search"));
            progressBar.setIndeterminate(false);
            progressBar.setDoubleValue(0);
            oldResults = talker.queryList();
            searchField.reloadData();
        }
    }

    private synchronized void startSearch() {
        // We stop the possible current thread, and we wait for its completion
        if (thread!=null) {
            thread.stopRunningImmediately();
            thread = null;
        }
        progressBar.setIndeterminate(true);
        progressBar.startAnimation(this);
        // Now, the action is a cancel until the texfield is modified
        searchButton.setTitle(Messages.string("Cancel"));
        mustSearch = false;
        // We create the new search
        thread = new ItemSearch();
        // Run the new one
        thread.start();
        updateDrawer();
    }
    
    public void startDownload(Object sender) {
        int row = resultList.selectedRow();
        if (row>=0) {
            SearchedItem item = dataSource.itemAtRow(row);
            talker.download(item);
            updateDrawer();
        }
    }

    public boolean windowShouldClose(NSNotification aNotification) {
        infoDrawer.close();
        extendedSearchDrawer.close();
        return true;
    }
    
    public void windowWillClose(NSNotification aNotification) {
        super.windowWillClose(aNotification);
        if (thread!=null)
            thread.stopRunning();
        Preferences.rememberColumnSizes(this, resultList);
    }
    
    public void tableViewSelectionDidChange(NSNotification aNotification) {
        int row = resultList.selectedRow();
        if (row>=0) {
            updateDrawer();
            infoDrawer.open();
        } else {
            infoDrawer.close();
        }
    }

    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        dataSource.tableViewDidClickTableColumn(tableView, tableColumn);
        updateDrawer();
    }
    
    public void updateDrawer() {
        int row = resultList.selectedRow();
        if (row>=0) {
            SearchedItem item = dataSource.itemAtRow(row);
            if (item==null) {
                infoDrawer.close();
                return;
            }
            codecField.setStringValue(item.codec()!=null ? item.codec() : "?");
            identifierField.setStringValue(item.size() + " " + item.hash());
            StringBuffer aliases = new StringBuffer(item.name());
            for(int i=0; i<item.aliases().size(); i++) {
                aliases.append("\n   ");
                aliases.append(item.aliases().get(i));
            }
            aliasesField.setStringValue(aliases.toString());
            if (downloadItemSource.item(item.size(), item.hash())==null) {
                downloadButton.setEnabled(true);
                downloadButton.setTitle(Messages.string("Start download"));
            } else {
                downloadButton.setEnabled(false);
                downloadButton.setTitle(Messages.string("Downloading"));
            }
        }
    }

    public int numberOfItemsInComboBox(NSComboBox aComboBox) {
        return oldResults.count();
    }

    public Object comboBoxObjectValueForItemAtIndex(NSComboBox aComboBox, int index) {
        if (index>=0 && index<oldResults.count())
            return oldResults.objectAtIndex(index).toString();
        else
            return null;
    }

    public int comboBoxIndexOfItem(NSComboBox aComboBox, String aString) {
        for(int i=0; i<oldResults.count(); i++) {
            SearchParser item = (SearchParser)oldResults.objectAtIndex(i);
            if (aString.equals(item.stringFor("")))
                return i;
        }
        return -1;
    }

    private void setSizeOn(String value, NSTextField test, NSPopUpButton type) {
        try {
            int n = Integer.parseInt(value);
            if (n>=1048576) {
                test.setIntValue(n/1048576);
                type.selectItemAtIndex(0);
            } else if (n>=1024) {
                test.setIntValue(n/1024);
                type.selectItemAtIndex(1);
            } else {
                test.setIntValue(n);
                type.selectItemAtIndex(2);
            }
        }
        catch(NumberFormatException e) {
            test.setStringValue("");
            type.selectItemAtIndex(0);
        }
    }
    
    public synchronized void comboBoxSelectionDidChange(NSNotification notification) {
        if (notification.object() != searchField) {
            return;
        }
        int row = searchField.indexOfSelectedItem();
        if (row>=0) {
            SearchParser item = (SearchParser)oldResults.objectAtIndex(row);
            currentSearch = item.searchNumber();
            if (item.isExtended()) {
                extendedButton.setState(NSCell.OnState);
                asAlbumField.setStringValue(item.stringFor("album"));
                asArtistField.setStringValue(item.stringFor("artist"));
                asFormatField.setStringValue(item.stringFor("format"));
                setSizeOn(item.stringFor("size", "<"), asMaxSizeField, asMaxSizeTypeButton);
                asMediaField.setStringValue(item.stringFor("type"));
                // NSPopUpButton asMinBitrateButton; /* IBOutlet */
                setSizeOn(item.stringFor("size", ">"), asMinSizeField, asMinSizeTypeButton);
                asTitleField.setStringValue(item.stringFor("title"));
                extendedSearchDrawer.open();
            } else {
                extendedButton.setState(NSCell.OffState);
                asAlbumField.setStringValue("");
                asArtistField.setStringValue("");
                asFormatField.setStringValue("");
                setSizeOn("zz", asMaxSizeField, asMaxSizeTypeButton);
                asMediaField.setStringValue("");
                // NSPopUpButton asMinBitrateButton; /* IBOutlet */
                setSizeOn("zz", asMinSizeField, asMinSizeTypeButton);
                asTitleField.setStringValue("");
                extendedSearchDrawer.close();
            }
            dataSource.setElements(new ArrayList());
            startSearch();
        }
    }

    public void comboBoxWillPopUp(NSNotification notification) {
        if (notification.object() != searchField)
            return;
        oldResults = talker.queryList();
        searchField.reloadData();
    }

    public void controlTextDidChange(NSNotification aNotification) {
        mustSearch = true;
        searchButton.setTitle(Messages.string("Search"));
    }
    
    private class ItemSearch extends WorkerThread {
        
        private int searchNumber;

        public ItemSearch() {
            searchNumber = currentSearch;
        }
        
        public void update(NSTimer timer) {
			List liste = (List)((NSArray)timer.userInfo()).objectAtIndex(0);
			int toGo = ((Number)((NSArray)timer.userInfo()).objectAtIndex(1)).intValue();
			if (liste.size()>0 || toGo==0) {
				progressBar.setIndeterminate(false);
				progressBar.setMaxValue(toGo==0 ? 100 : toGo+liste.size());
				progressBar.setDoubleValue(toGo==0 ? 100 : liste.size());
			}
			dataSource.setElements(liste);
			updateDrawer();
        }
        
        protected boolean work() {
            if (searchNumber!=currentSearch)
                return false;
            Object[] res = talker.queryResult(searchNumber);
            int toGo = ((Integer)res[0]).intValue();
            List liste = (List)res[1];
            if (!isStopping()) {
                synchronized(SearchWindowController.this) {
					doTask("update", new NSArray(new Object[]{liste, new Integer(toGo)}));
                }
            }
            return toGo>0;
        }
    }
}

// $Log: SearchWindowController.java,v $
// Revision 1.13  2002/07/30 00:16:37  fortun
// New extendSearch() method
//
// Revision 1.12  2002/07/22 00:30:23  fortun
// NSArrays are now Lists
//
// Revision 1.11  2002/07/21 18:35:17  fortun
// Extended search implemented.
// Drawer now has a good size
//
// Revision 1.10  2002/07/08 21:43:57  fortun
// Drawer is closed before the window, otherwise the application won't quit
//
// Revision 1.9  2002/07/07 22:42:59  fortun
// Update of the drawer when sorting
//
// Revision 1.8  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.7  2002/07/07 19:38:45  fortun
// Now handles the selection of the header
//
// Revision 1.6  2002/06/30 21:05:17  fortun
// Remembers window pos & size
//
// Revision 1.5  2002/06/29 17:54:56  fortun
// Bugs corrected, it should now be very stable
//
// Revision 1.4  2002/06/29 11:15:53  fortun
// Old thread is now killed before launching the new one
// ProgressBar is now stoppped before its type is modified
//
// Revision 1.3  2002/06/02 12:02:15  fortun
// Search thread now properly stopped
//
// Revision 1.2  2002/05/27 22:05:22  fortun
// Delegates sorting to the dataSource
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
