//
//  SortedTableView.java
//  CocoDonkey
//  $Id: TableViewSorter.java,v 1.7 2003/01/03 00:01:39 fortun Exp $
//
//  Created by Fred Bonzoun on Mon May 27 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import java.util.*;

public class TableViewSorter {

    NSTableView tableView; /* IBOutlet */
    
    private List columns = new ArrayList();
    private List selectors = new ArrayList();
    private int listOrdering = 0;
    private List unorderedList = new ArrayList();
    private boolean isListSorted = true;
    private List list = new Vector();
    private Object lastSelection;

    public TableViewSorter() {
    }
    
    public void addSortedColumn(String name, Comparator selector) {
        NSTableColumn column = tableView.tableColumnWithIdentifier(name);
        if (column==null)
            throw new IllegalArgumentException("Non-existent column name: " + name);
        SortedTableHeaderCell cell = new SortedTableHeaderCell(column.headerCell().stringValue());
        cell.setAlignment(column.headerCell().alignment());
        column.setHeaderCell(cell);
        columns.add(column);
        selectors.add(selector);
    }

    public synchronized void sortOnName(String name) {
        lastSelection = currentSelection();
        // We mark the old sorted column (if any) as not sorted
        boolean wasSorted = (listOrdering!=0);
        if (wasSorted) {
            NSTableColumn column = (NSTableColumn)columns.get(Math.abs(listOrdering)-1);
            ((SortedTableHeaderCell)column.headerCell()).setOrdering(0);
        }
        // The list will have to be sorted
        isListSorted = false;
        // We look for the column in our list of sorted columns
        boolean found = false;
        for(int i=0; i<columns.size(); i++) {
            NSTableColumn column = (NSTableColumn)columns.get(i);
            if (column.identifier().equals(name)) {
                i++; // For us, the first column is 1
                // We invert the sort order if the column has not changed
                listOrdering = (listOrdering==i ? -i : listOrdering==-i ? 0 : i);
                found = true;
                break;
            }
        }
        if (!found)
            return;
        // We notify the column that it has a new sort order
        boolean isSorted = (listOrdering!=0);
        if (isSorted) {
            NSTableColumn column = (NSTableColumn)columns.get(Math.abs(listOrdering)-1);
            ((SortedTableHeaderCell)column.headerCell()).setOrdering(listOrdering);
        }
        //sortList(); // Sorting is directly done in the list() getter
        if (wasSorted!=isSorted)
            restoreSelection(lastSelection);
        tableView.headerView().setNeedsDisplay(true);
        tableView.setNeedsDisplay(true);
    }
    
    public synchronized void setList(List list) {
        Object oldSelection = (listOrdering!=0 ? currentSelection() : null);
        List oldList = unorderedList;
        unorderedList = new ArrayList(list);
        isListSorted = false;
        restoreSelection(oldSelection);
        if (oldList==null || oldList.size() != list.size())
            tableView.noteNumberOfRowsChanged();
        tableView.setNeedsDisplay(true);
    }
    
    public final List list() {
        if (listOrdering==0) {
            lastSelection = null;
            return unorderedList;
        } else {
            if (!isListSorted)
                sortList();
            lastSelection = null;
            return list;
        }
    }

    private Object currentSelection() {
        List l = (listOrdering==0 ? unorderedList : list);
        int i = tableView.selectedRow();
        if (i>=0 && i<l.size()) {
            return l.get(i);
        } else
            return null;
    }

    private void restoreSelection(Object o) {
        if (o==null)
            return;
        if (o.equals(currentSelection()))
            return;
        List l = (listOrdering==0 ? unorderedList : list);
        int i = l.indexOf(o);
        if (i>=0) {
            tableView.selectRow(i, false);
            tableView.scrollRowToVisible(i);
        } else {
            for(i=0; i<l.size(); i++)
                System.out.println("  " + l.get(i));
        }
    }
    
    public synchronized void addObject(Object item) {
        isListSorted = false;
        tableView.setNeedsDisplay(true);
        tableView.noteNumberOfRowsChanged();
        unorderedList.add(item);
    }

    public synchronized void updateObject(Object item) {
        isListSorted = false;
        tableView.setNeedsDisplay(true);
    }
    
    public synchronized void sortList() {
        isListSorted = true;
        if (listOrdering==0) {
            list = unorderedList;
        } else {
            Object oldSelection = currentSelection();
            // hack because the first sort has no selection set
            if (oldSelection==null)
                oldSelection = lastSelection;
            Comparator sorter = (Comparator)selectors.get(Math.abs(listOrdering)-1);
            Object[] temp = unorderedList.toArray();
            Arrays.sort(temp, sorter);
            if (listOrdering>=0) {
                list = Arrays.asList(temp);
            } else {
                list = new ArrayList();
                for(int i=temp.length-1; i>=0; i--)
                    list.add(temp[i]);
            }
            restoreSelection(oldSelection);
        }
    }

    public void tableViewDidClickTableColumn(NSTableView tableView, NSTableColumn tableColumn) {
        if (tableView==this.tableView) {
            String name = (String)tableColumn.identifier();
            sortOnName(name);
        }
    }
}

// $Log: TableViewSorter.java,v $
// Revision 1.7  2003/01/03 00:01:39  fortun
// Now maintains the current selection, if any
//
// Revision 1.6  2002/07/25 23:29:33  fortun
// Now it updates the display itself
//
// Revision 1.5  2002/07/22 00:30:23  fortun
// NSArrays are now Lists
//
// Revision 1.4  2002/07/08 21:42:23  fortun
// Bug in the display solved
//
// Revision 1.3  2002/07/07 20:07:28  fortun
// Simpler handling of the selection of the header
//
// Revision 1.2  2002/07/07 19:36:02  fortun
// Now handles the selection of the header
//
// Revision 1.1  2002/05/27 22:06:08  fortun
// Handles the sorting of a list and the corresponding table headers
//
