//
//  WorkerThread.java
//  CocoDonkey
//  $Id: WorkerThread.java,v 1.9 2002/07/14 22:43:40 fortun Exp $
//
//  Created by Fred Bonzoun on Fri May 17 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

import net.bonzoun.cocodonkey.DonkeyTalker;

public class WorkerThread extends Thread  implements DonkeyTalker.Listener {

    private static NSMutableArray runningThreads = new NSMutableArray();
    
    private boolean threadShouldBeRunning = true;
    private boolean threadIsRunning = false;
    private boolean threadIsStopping = false;
    private boolean threadWillStop = false;
    private Object monitor = new Object();
    private NSRunLoop runLoop;


    public WorkerThread() {
        synchronized(runningThreads) {
            runningThreads.addObject(this);
        }
		runLoop = NSRunLoop.currentRunLoop();
	}
    
    public void wakeUp() {
        synchronized(monitor) {
            monitor.notifyAll();
        }
    }
    
	protected void doTask(String task, Object param) {
        if (threadShouldBeRunning && !threadWillStop) {
            NSSelector sel = new NSSelector(task, new Class[]{NSTimer.class});
            NSTimer timer = new NSTimer(0., this, sel, param, false);
            runLoop.addTimerForMode(timer, NSRunLoop.DefaultRunLoopMode);
        }
	}
	
    public static boolean prepareForExit() {
        // We will try for 5 seconds to stop the threads
        for(int i=0; i<50; i++) {
            boolean allStoppped = true;
            synchronized(runningThreads) {
                for(int j=0; j<runningThreads.count(); j++) {
                    WorkerThread thread = (WorkerThread)runningThreads.objectAtIndex(j);
                    if (thread.threadWillStop)
                        allStoppped = false;
                    else if (thread.threadShouldBeRunning) {
                        thread.stopRunning();
                        allStoppped = false;
                    }
                }
            }
            if (allStoppped)
                return true;
            try {
                Thread.sleep(100);
            } catch(InterruptedException e) {}
        }
        return false;
    }

    public boolean isRunning() {
        return threadIsRunning;
    }

    public boolean isStopping() {
        return threadIsStopping;
    }
    
    public void stopRunning() {
        if (threadShouldBeRunning && !threadWillStop) {
            threadWillStop = true;
            new Stopper().start();
        }
    }

    public void stopRunningImmediately() {
        if (threadShouldBeRunning) {
            if (threadIsStopping) {
                // We wait for the other thread stopping
                synchronized(monitor) {
                    try {
                        monitor.wait(1000);
                    } catch(InterruptedException e) {}
                }
            } else {
                // We stop it
                threadWillStop = true;
                threadIsStopping = true;
                // We guaranty to be alone to test
                synchronized (monitor) {
                    if (threadShouldBeRunning && threadIsRunning) {
                        threadShouldBeRunning = false;
                        // We wake the thread
                        monitor.notifyAll();
                        // Still there ? We will wait for it to finish
                        if (threadIsRunning) {
                            try {
                                monitor.wait(1000);
                            } catch(InterruptedException e) {}
                        }
                    } else {
                        threadShouldBeRunning = false;
                    }
                }
                threadIsStopping = false;
                threadWillStop = false;
            }
        }
    }
    
    public void stopListening() {
        stopRunning();
    }
    
    protected boolean work() throws Exception {
        return false;
    }

    protected void dies() throws Exception {
    }

    protected int refreshInterval() {
        int r = Preferences.refreshInterval();
        if (r<=0)
            return 5;
        else
            return r;
    }
    
    public void run() {
        synchronized (monitor) {
            //System.out.println("[" + new java.util.Date() + "] Thread started: " + getClass().getName());
            threadIsRunning = true;
            // The actual work
            while (threadShouldBeRunning) {
                // The actual work
                int ar0 = NSAutoreleasePool.push();
                try {
                    if (!work())
                        break;
                    monitor.wait(refreshInterval()*1000);
                }
                catch(Throwable e) {
                    e.printStackTrace();
                    break;
                }
                finally {
                    NSAutoreleasePool.pop(ar0);
                }
            }
            int ar1 = NSAutoreleasePool.push();
            try {
                dies();
            }
            catch(Throwable e) {
                e.printStackTrace();
            }
            finally {
                //System.out.println("[" + new java.util.Date() + "] Thread ended: " + getName() + "/" + getClass());
                threadIsRunning = false;
                monitor.notifyAll();
                NSAutoreleasePool.pop(ar1);
                synchronized(runningThreads) {
                    runningThreads.removeObject(this);
                }
            }
        }
    }

    private class Stopper extends Thread {
        public void run() {
            stopRunningImmediately();
        }
    }
}

// $Log: WorkerThread.java,v $
// Revision 1.9  2002/07/14 22:43:40  fortun
// New threadWillStop instance variable to better handle stopping
//
// Revision 1.8  2002/07/09 22:09:02  fortun
// New wakeUp() method to wake up the thread
//
// Revision 1.7  2002/06/29 17:57:05  fortun
// Added a dotask() function to send an event to the main thread
// (usefull when you want to deal with the GUI)
//
// Revision 1.6  2002/06/29 11:21:03  fortun
// New stopRunningImmediately() method to wait for the thread completion
//
// Revision 1.5  2002/06/14 00:38:59  fortun
// Implements the new DonkeyTalker.Listener interface to be stopped when the talker stops
//
// Revision 1.4  2002/06/02 17:54:11  fortun
// New "prepareForExit" function to properly stop a thread
//
// Revision 1.3  2002/06/02 11:59:02  fortun
// Stop of the thread is done in a separate thread, so that it never fails
//
// Revision 1.2  2002/06/01 15:16:00  fortun
// Better thread-safe way to stop the Thread
//
// Revision 1.1.1.1  2002/05/21 21:26:10  fortun
//
//
