//
//  GenericItem.java
//  CocoDonkey
//  $Id: GenericItem.java,v 1.9 2003/01/03 00:02:31 fortun Exp $
//
//  Created by Fred Bonzoun on Sun May 12 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

import com.apple.cocoa.foundation.*;
import java.util.*;

public abstract class GenericItem {

    protected int id;
    protected String name;
    protected int size;
    protected String hash;
    protected int nbClients;
    
    protected boolean complete;

    private static final PrintfFormat pfInt = new PrintfFormat("%02d");
    private static final PrintfFormat pfFloat = new PrintfFormat("%3.1f");

    private static final int SEC_PER_DAY = 86400;
    private static final int SEC_PER_HOUR = 3600;
    private static final int SEC_PER_MIN = 60;

    /*package*/ GenericItem(int id, String name) {
        this.id = id;
        this.name = name;
        complete = false;
    }

    public void setNbClients(int nb) {
        this.nbClients=nb;
    }

    public final int id() {
        return id;
    }

    public final String name() {
        return name;
    }

    public final int size() {
        return size;
    }

    public final String prettySize() {
        return displaySize(size);
    }

    public final int nbClients() {
        return nbClients;
    }

    public final String hash() {
        return hash;
    }

    public static boolean sameList(List list1, List list2) {
        if (list1.size()!=list2.size())
            return false;
        for(int i=0; i<list1.size(); i++) {
            GenericItem item1 = (GenericItem)list1.get(i);
            GenericItem item2 = (GenericItem)list2.get(i);
            if (!item1.equals(item2))
                return false;
        }
        return true;
    }

    protected boolean idIsConstant() {
        return true;
    }
    
    public boolean equals(Object o) {
        if (!(o instanceof GenericItem))
            return false;
        GenericItem item = (GenericItem)o;
        if (idIsConstant() && id()!=item.id())
            return false;
        if (name()!=null && item.name!=null && !name().equals(item.name()))
            return false;
        if (size()>0 && item.size()>0 && size()!=item.size())
            return false;
        if (hash()!=null && item.hash()!=null && !hash().equals(item.hash()))
            return false;
        return true;
    }

    public String toString() {
        String s = name() + " / " + size() + " " + hash();
        if (idIsConstant())
            s = "[" + id() + "] " + s;
        return s;
    }
    
    public static abstract class MyComparator implements java.util.Comparator {
        public boolean equals(Object o) {
            return equals(o);
        }
    }
    
    public static class CompareName extends MyComparator {
        public final int compare(Object o1, Object o2) {
            GenericItem item1 = (GenericItem)o1;
            GenericItem item2 = (GenericItem)o2;
            if (item1.name()!=null)
                return item1.name().compareToIgnoreCase(item2.name());
            else
                return 1;
        }
    }

    public static class CompareSize extends MyComparator {
        public final int compare(Object o1, Object o2) {
            GenericItem item1 = (GenericItem)o1;
            GenericItem item2 = (GenericItem)o2;
            if (item1.size()<item2.size())
                return -1;
            else if (item1.size()==item2.size())
                return 0;
            else
                return 1;
        }
    }

    public static class CompareNbClients extends MyComparator {
        public final int compare(Object o1, Object o2) {
            GenericItem item1 = (GenericItem)o1;
            GenericItem item2 = (GenericItem)o2;
            if (item1.nbClients()<item2.nbClients())
                return -1;
            else if (item1.nbClients()==item2.nbClients())
                return 0;
            else
                return 1;
        }
    }
    
    public static String displaySize(int n) {
        if (n<2048)
            return "" + n + " B";
        float f = (float)n;
        f /= 1024;
        if (f<2048)
            return "" + (Math.round(f*10.)/10.) + " kB";
        f /= 1024;
        if (f<2048)
            return "" + (Math.round(f*10.)/10.) + " MB";
        f /= 1024;
        return "" + (Math.round(f*10.)/10.) + " GB";
    }

    /** Tries to display an approximation of 10%, always between 5% and 20% **/
    public static String displayTime(int n) {
        if (n < 0) {
            return "Unknown";
        } else if (n < 50) {
            // Below 50s: 1s, 2s, ... up from 2%
            return  "~ " + n + " s";
        } else if (n <= 5*SEC_PER_MIN) {
            // From 50s to 5m: 1m 10s, 1m 20s, ... down from 20% to 3.3%
            int min = n / SEC_PER_MIN;
            int sec = ((n%SEC_PER_MIN) / 10) * 10;
            if (sec>0)
                return  "" + min + " m " + sec + " s";
            else
                return  "" + min + " m";
        } else if (n <= 20*SEC_PER_MIN) {
            // From 5 to 20 minutes: 1m, 2m... down from 20% to 5%
            int min = (n+SEC_PER_MIN/2) / SEC_PER_MIN;
            return  "~ " + min + " m";
        } else if (n <= SEC_PER_HOUR) {
            // From 20 minutes to 1 hour : 5m; 10m, ... down from 25% to 8.4%
            int min = ((n+SEC_PER_MIN/2) / SEC_PER_MIN / 5) * 5;
            return  "~ " + min + " m";
        } else if (n < 5*SEC_PER_HOUR) {
            // From 1 to 5 hour: 1h 10m, 1h 20m, ... down from 17% to 3.3%
            int hours = n / SEC_PER_HOUR;
            int min = (((n%SEC_PER_HOUR)+SEC_PER_MIN/2) / SEC_PER_MIN / 10) * 10;
            if (min>0)
                return "" + hours + " h " + min + " m";
            else
                return "" + hours + " h";
        } else if (n <= SEC_PER_DAY) {
            // From 5 to 24 hours: 1h, 2h, ... down from 20% to 4%
            int hours = (n+SEC_PER_HOUR/2) / SEC_PER_HOUR;
            return  "" + hours + " h";
        } else if (n < 5*SEC_PER_DAY) {
            // From 1 to 5 days: 2d 4h, 2d 8h, ... down from 17% up to 3.3%
            int days = n / SEC_PER_DAY;
            int hours = (((n%SEC_PER_DAY)+SEC_PER_HOUR/2) / SEC_PER_HOUR / 4) * 4;
            if (hours>0)
                return "" + days + " d " + hours + " h";
            else
                return "" + days + " d";
        } else {
            // Above 5 days: 5 d, 6 d, ... down from 20% to 0%
            int days = (n+SEC_PER_DAY/2) / SEC_PER_DAY;
            return "" + days + " d";
        }
    }
}

// $Log: GenericItem.java,v $
// Revision 1.9  2003/01/03 00:02:31  fortun
// equals() now works
// toString() now implemented
//
// Revision 1.8  2002/07/22 00:30:22  fortun
// NSArrays are now Lists
//
// Revision 1.7  2002/07/08 22:01:09  fortun
// Bug in displayTime if second term is 0 (displayed but should not)
//
// Revision 1.6  2002/07/07 12:43:20  fortun
// Sorting is now case insensitive
//
// Revision 1.5  2002/06/09 22:45:29  fortun
// Finer algorithm for displayTime()
//
// Revision 1.4  2002/06/09 15:44:03  fortun
// Simplified display for the timeLeft()
//
// Revision 1.3  2002/06/08 12:20:59  fortun
// Estimation of the time remaining
//
// Revision 1.2  2002/05/27 22:08:30  fortun
// Added some comparators
// Added an 'equal' method
// Added the 'sameList' method to compare two lists of GenericItems
//
// Revision 1.1.1.1  2002/05/21 21:26:13  fortun
//
//
