//
//  RateCalculator.java
//  CocoDonkey
//  $Id: RateCalculator.java,v 1.4 2002/07/21 18:39:32 fortun Exp $
//
//  Created by Fred Bonzoun on Sat Jun 08 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

import java.util.*;

/**
 * This class is intended to calculate a remaining time for a download.
 * For that, it uses instant download rates and makes complicated averages
 * on them to finally get a better approximation of the remaining time
 * than what the instant download rate can give.
 * A first calculation is done with a floating average of the rate on a
 * one minute length (MINUTE_LENGTH*MINUTE_PRECISION gives the length of a minute
 * for the calculation, the unite being the second).
 * A second estimation uses a one hour length for the floating average (one hour
 * is HOUR_LENGTH*HOUR_PRECISION long, the unite being the second).
 * The best of these two calculations is supposed to be the best bet for the current rate.
 * The addRate() method is used to add a rate. It can be called as often as possible as
 * the current class only uses the required data (MINUTE_PRECISION determines the amount
 * of data that will be retained).
 **/
public class RateCalculator {

    /** Minimum amount of seconds between two kept "minute" rates **/
    private static final int MINUTE_PRECISION = 2;
    /** Number of rates kept for a "minute" **/
    private static final int MINUTE_LENGTH = 20;
    /** Number of rates kept for an "hour" **/
    private static final int HOUR_LENGTH = 60;
    /** Minimum amount of seconds between two kept "hour" rates **/
    private static final int HOUR_PRECISION = 60;

    private RateKeeper lastMinute;
    private int minuteAddedCount = 0;
    private RateKeeper lastHour;
    
    public RateCalculator() {
        lastMinute = new RateKeeper(MINUTE_PRECISION, MINUTE_LENGTH);
        lastHour = new RateKeeper(HOUR_PRECISION, HOUR_LENGTH);
    }

    public void addRate(float rate) {
        int nbAdded = lastMinute.addRate(rate);
        minuteAddedCount += nbAdded;
        if (minuteAddedCount>=HOUR_PRECISION/MINUTE_PRECISION) {
            minuteAddedCount -= HOUR_PRECISION/MINUTE_PRECISION;
            lastHour.addRate(lastMinute.meanRate());
        }
    }

    public float meanRate() {
        float minuteRate = lastMinute.meanRate();
        float hourRate = lastHour.meanRate();
        return Math.max(minuteRate, hourRate);
    }
    
    class RateKeeper {
        private int precision;
        private int maxSize;
        private Vector rateList = new Vector();
        private float rateTotal = 0;
        private long lastAddTime = System.currentTimeMillis();

        RateKeeper(int precision, int maxSize) {
            this.precision = precision;
            this.maxSize = maxSize;
        }
        
        public synchronized int addRate(float rate) {
            long t = System.currentTimeMillis();
            if (t-lastAddTime > (long)precision*1000) {
                // We need to keep this one
                int nb = 0;
                for(int i=0; t-lastAddTime > (long)precision*1000 && i<maxSize; i++) {
                    lastAddTime += (long)precision*1000;
                    nb++;
                    addOneRate(rate);
                }
                lastAddTime = t;
                return nb;
            } else {
                return 0;
            }
        }

        private synchronized void addOneRate(float rate) {
            if (rate<0 || rate>1000)
                rate=0;
            // We need to keep this one
            rateTotal += rate;
            rateList.add(0, new Float(rate));
            // We keep a given size to the list
            if (rateList.size()>maxSize) {
                Number r = (Number)rateList.remove(rateList.size()-1);
                rateTotal -= r.floatValue();
            }
        }
        
        public float meanRate() {
            if (rateList.size()==0)
                return -1;
            else
                return rateTotal / rateList.size();
        }

        public boolean isFull() {
            return rateList.size()==maxSize;
        }
    }
}

// $Log: RateCalculator.java,v $
// Revision 1.4  2002/07/21 18:39:32  fortun
// Proper creator set
//
// Revision 1.3  2002/07/08 21:40:29  fortun
// Comment added
//
// Revision 1.2  2002/06/14 00:41:52  fortun
// Sorter timing
//
// Revision 1.1  2002/06/09 15:44:24  fortun
// Calculates rates, with means over the time
//
