//
//  StringParser.java
//  CocoDonkey
//	$Id: StringParser.java,v 1.5 2003/01/01 17:35:01 fortun Exp $
//
//  Created by Fred Bonzoun on Tue Jul 09 2002.
//  Copyright (c) 2002 Bonzoun. All rights reserved.
//
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package net.bonzoun.cocodonkey;

public class StringParser {

    public static final String WHITESPACE = " \t\r\n";
    public static final String INTEGER = "0123456789";
    public static final String HEXA = "0123456789abcdefABCDEF";
    public static final String FLOAT = INTEGER  + ".";
    
    private String data;
    private int start;
    private int end;

    public StringParser(String s) {
        data = s;
        start = 0;
        end = s.length()-1;
    }

    public static boolean stringContainsOnly(String s, String chars) {
        int nb = s.length();
        for(int i=0; i<nb; i++) {
            if (chars.indexOf(s.charAt(i))<0)
                return false;
        }
        return true;
    }
    
    /** Skips the next empty chars **/
    private void skipNextEmpty() {
        for(int i=start; i<=end; i++) {
            if (WHITESPACE.indexOf(data.charAt(i))<0) {
                start = i;
                return;
            }
        }
        start = end+1;
    }

    /** Skips the last empty chars **/
    private void skipLastEmpty() {
        for(int i=end; i>=start; i--) {
            if (WHITESPACE.indexOf(data.charAt(i))<0) {
                end = i;
                return;
            }
        }
        end = start-1;
    }

    public boolean hasMoreData() {
        return start<=end;
    }
    
    /** Returns the next non empty chars **/
    public String getNextString() {
        return getNextStringUntil(WHITESPACE);
    }

    public int getNextInt(int def) {
        return parseInt(getNextString(), def);
    }

    public float getNextFloat(int def) {
        return parseFloat(getNextString(), def);
    }
    
    /** Returns the next non empty chars until any char in delim is met, skipping the head withespaces **/
    public String getNextStringUntil(String delim) {
        skipNextEmpty();
        for(int i=start; i<=end; i++) {
            if (delim.indexOf(data.charAt(i))>=0) {
                String s = (i>start ? data.substring(start, i) : "");
                start = i;
                return s;
            }
        }
        String s = (start<=end ? data.substring(start, end+1) : "");
        start = end+1;
        return s;
    }
    
    public char getNextChar() {
        char c = testNextChar();
        start++;
        return c;
    }

    public char testNextChar() {
        skipNextEmpty();
        if (start<=end)
            return data.charAt(start);
        else
            return '\0';
    }
    
    public int getNextIntUntil(String delim, int def) {
        return parseInt(getNextStringUntil(delim + WHITESPACE), def);
    }

    /** Skips the next chars until delim is met **/
    public boolean goAfter(char delim) {
        for(int i=start; i<=end; i++) {
            if (delim == data.charAt(i)) {
                start = i+1;
                return true;
            }
        }
        start = end+1;
        return false;
    }

    /** Skips the next chars until delim is met **/
    public boolean goBefore(char delim) {
        for(int i=end; i>=start; i--) {
            if (delim == data.charAt(i)) {
                end = i-1;
                return true;
            }
        }
        end = start-1;
        return false;
    }
    
    /** Returns the last non empty chars **/
    public String getLastString() {
        return getLastStringUntil(WHITESPACE);
    }

    public int getLastInt(int def) {
        return parseInt(getLastString(), def);
    }

    public float getLastFloat(int def) {
        return parseFloat(getLastString(), def);
    }
    
    /** Returns the last non empty chars until any char in delim is met, skipping the head withespaces **/
    public String getLastStringUntil(String delim) {
        skipLastEmpty();
        for(int i=end; i>=start; i--) {
            if (delim.indexOf(data.charAt(i))>=0) {
                String s = (i<end ? data.substring(i+1, end+1) : "");
                end = i;
                return s;
            }
        }
        String s = (start<=end ? data.substring(start, end+1) : "");
        end = start-1;
        return s;
    }

    public char getLastChar() {
        char c = testLastChar();
        end--;
        return c;
    }

    public char testLastChar() {
        skipLastEmpty();
        if (start<=end)
            return data.charAt(end);
        else
            return '\0';
    }
    
    public int getLastIntUntil(String delim, int def) {
        return parseInt(getLastStringUntil(delim + WHITESPACE), def);
    }

    public String getRemainingString() {
        skipNextEmpty();
        skipLastEmpty();
        if (start<=end) {
            return data.substring(start, end+1);
        } else {
            return "";
        }
    }
    
    /** Converts a String to an int, without failing **/
    public static int parseInt(String s, int def) {
        try {
            return Integer.parseInt(s);
        }
        catch(Exception e) {
            return def;
        }
    }

    /** Converts a String to an float, without failing **/
    public static float parseFloat(String s, int def) {
        try {
            return Float.parseFloat(s);
        }
        catch(Exception e) {
            return def;
        }
    }
}

// $Log: StringParser.java,v $
// Revision 1.5  2003/01/01 17:35:01  fortun
// Added stringContainsOnly()
//
// Revision 1.4  2002/08/21 21:49:34  fortun
// NUMBER becomes INTEGER
//
// Revision 1.3  2002/07/22 00:30:03  fortun
// testLastChar() method added
//
// Revision 1.2  2002/07/21 18:40:49  fortun
// New getNextChar/getLastChar and testNextChar functions
//
// Revision 1.1  2002/07/09 22:14:34  fortun
// Fast parser
//
